@@ -7,7 +7,7 @@ use warnings;
 use Data::Clone;
 use SHARYANTO::Color::Util qw(mix_2_rgb_colors);
 
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 our %color_themes = (
 
@@ -63,7 +63,7 @@ _
 
             num_data    => '66ffff',
             str_data    => undef,
-            date_data   => undef,
+            date_data   => 'cccc00',
             bool_data   => sub {
                 my ($self, %args) = @_;
 
@@ -78,6 +78,19 @@ my $ng = clone($color_themes{default_gradation});
 $ng->{colors}{border} = '666666';
 $color_themes{default_nogradation} = $ng;
 
+my $dgw = clone($color_themes{default_gradation});
+$dgw->{colors}{header_bg} = 'cccccc';
+$dgw->{data}{default_border1} = '000000';
+$dgw->{data}{default_border2} = 'cccccc';
+$dgw->{colors}{num_data}  = '006666',
+$dgw->{colors}{date_data} = '666600';
+$dgw->{colors}{bool_data} = sub {
+    my ($self, %args) = @_;
+
+    $args{orig_data} ? '00cc00' : 'cc0000';
+},
+$color_themes{default_gradation_whitebg} = $dgw;
+
 1;
 # ABSTRACT: Default color themes
 
@@ -93,7 +106,7 @@ Text::ANSITable::ColorTheme::Default - Default color themes
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 AUTHOR
 
@@ -127,6 +140,16 @@ C<border2_bg> to set background RGB colors.
 
 
 
+=item * default_gradation_whitebg (Default for 256-color+ terminal (black background))
+
+
+Border color has gradation from top to bottom. Accept arguments C<border1> and
+C<border2> to set first (top) and second (bottom) foreground RGB colors. Colors
+will fade from the top color to bottom color. Also accept C<border1_bg> and
+C<border2_bg> to set background RGB colors.
+
+
+
 =item * default_nogradation (Default for 256-color+ terminal (black background))
 
 
